package com.flyfish.framework.domain.po;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.flyfish.framework.domain.base.AuditDomain;
import com.flyfish.framework.domain.base.IUser;
import com.flyfish.framework.enums.UserStatus;
import com.flyfish.framework.enums.UserType;
import com.flyfish.framework.validation.spi.ConditionalGroup;
import lombok.*;
import org.springframework.data.annotation.Reference;
import org.springframework.data.annotation.Transient;
import org.springframework.data.mongodb.core.index.Indexed;
import org.springframework.data.mongodb.core.mapping.Document;
import org.springframework.data.relational.core.mapping.Table;

import java.time.LocalDateTime;
import java.util.List;

@Document
@Table
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@EqualsAndHashCode(callSuper = true)
@ConditionalGroup(codeless = true)
public class User extends AuditDomain implements IUser {

    private static final long serialVersionUID = -960011918745179950L;

    /**
     * 用户类型
     */
    private UserType type;

    /**
     * 用户状态
     */
    private UserStatus status;

    /**
     * 冗余的电话号码
     */
    private String phone;

    /**
     * 用户名
     */
    @Indexed(unique = true)
    private String username;

    /**
     * 密码
     */
    @JsonProperty(access = JsonProperty.Access.WRITE_ONLY)
    private String password;

    /**
     * 是否启用
     */
    private Boolean enable;

    /**
     * 能否登录移动端
     */
    private Boolean app;

    /**
     * 有效期
     */
    @JsonFormat(pattern = "yyyy-MM-dd")
    private LocalDateTime validDate;

    /**
     * 可操作校区
     */
    @Reference(Department.class)
    private List<Department> departments;

    /**
     * 所属角色
     */
    @Reference(Role.class)
    private List<Role> roles;

    /**
     * 微信openId
     */
    private String openId;

    /**
     * 查询冗余，标记用户信息
     */
    @Transient
    private Object detail;

    /**
     * 当前用户的鉴权标记
     */
    private String authority;

    /**
     * 错误次数，防止恶意登录
     */
    private Integer errorCount = 0;

    /**
     * 上次登录日期
     */
    private LocalDateTime lastTime;

    @Override
    public User toUser() {
        return this;
    }
}
